<?php

namespace smusatov\bzb2u\Render;

use smusatov\bzb2u\Job;
use smusatov\bzb2u\Label;
use smusatov\bzb2u\Render\Text\Line;

class Text
{
    protected Label $label;
    /** @var Line[] */
    protected array $lines = [];

    public function __construct(Label $label)
    {
        $this->label = $label;
    }

    public function addLine(Line $line)
    {
        $this->lines[] = $line;
    }

    public function getJob(): Job
    {
        $im = new \Imagick();
        $im->setGravity(\Imagick::GRAVITY_NORTHEAST);
        $im->setType(\Imagick::IMGTYPE_BILEVEL);
        $im->setAntiAlias(false);
        $im->setFormat('png');
        $im->setCompression(\Imagick::COMPRESSION_NO);
        $im->setBackgroundColor(new \ImagickPixel('#ffffff'));
        $im->newImage(intval(ceil($this->label->getWidth() * 8)), intval(ceil($this->label->getHeight() * 8)), new \ImagickPixel('#ffffff'), 'png');

        $widthLine = $this->label->getWidth(true);
        $heightLine = floor(($this->label->getHeight(true) - 8) / count($this->lines));
        $offsetTop = 4;
        foreach($this->lines as $line) {
            $line->setSize($widthLine, $heightLine);
            $line = $line->render();
            $im->compositeImage($line, \Imagick::COMPOSITE_DEFAULT, floatval(($widthLine - $line->getImageWidth()) / 2), floatval(($heightLine - $line->getImageHeight()) / 2 + $offsetTop));

            $offsetTop += $heightLine;
        }

        $job = new Job($this->label);
        $iterator = $im->getPixelIterator();
        foreach($iterator as $y => $row) {

            foreach($row as $x => $pixel) {
                $job->setPixel($x, $y, $pixel->getHSL()['luminosity'] <= 0.45);
            }
            $iterator->syncIterator();
        }

        return $job;
    }

}
